class LLMFamilies:
    """Model identifier with extensibility support."""

    ANTHROPIC = "anthropic"
    OPENAI = "openai"
    GOOGLE = "google"
    OLLAMA = "ollama"
    DEEPSEEK = "deepseek"
    MISTRAL = "mistral"
    META = "meta"
    COHERE = "cohere"
    XAI = "xai"
    MOONSHOT = "moonshot"
