"""Spaik SDK - SDK for building various kinds of agentic + AI solutions."""

from .agent.base_agent import BaseAgent
from .models.llm_config import LLMConfig
from .models.llm_model import LLMModel
from .models.providers.provider_type import ProviderType
from .thread.models import MessageBlock, MessageBlockType, ThreadMessage
from .thread.thread_container import ThreadContainer

__all__ = [
    "BaseAgent",
    "LLMConfig",
    "LLMModel",
    "ProviderType",
    "ThreadContainer",
    "ThreadMessage",
    "MessageBlock",
    "MessageBlockType",
]

__version__ = "0.0.1"
